@@ -5,7 +5,7 @@ from pyeoskit import eosapi, wallet
 #import your account private key here
 wallet.import_key('mywallet', '5K463ynhZoCDDa4RDcr63cUwWLTnKqmdcoTKTHBjqoKfv4u5V7p')
 
-eosapi.set_node('https://api.eosn.com')
+eosapi.set_node('https://api.eosn.io')
 info = eosapi.get_info()
 print(info)
 args = {
